﻿using GenLib.Entity;
using GenLib.Methods;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;

namespace DocsGenTest.Unit
{
    [TestClass]
    public class MethodsLibTest
    {
        [TestMethod]
        [DataRow("10.3.2022", "dd. MMMM yyyy", "10. března 2022")]
        [DataRow("10.3.2022", "MM-dd-yyyy HH:mm", "10-03-2022 00:00")]
        [DataRow("16.5.2015 05:50:06", "yyyy'-'MM'-'dd'T'HH':'mm':'ss", "2015-05-16T05:50:06")]
        [DataRow("2015-05-01T05:50:06", "MMMM dd", "května 01")]
        public void MethodFormatDate_CorrectFormat_True(string date, string format, string expectedValue)
        {
            Parameter parameter = new Parameter(format);

            string currentValue = MethodsLib.MethodFormatDate(date, parameter);

            Assert.AreEqual(expectedValue, currentValue);
        }

        [TestMethod]
        [ExpectedException(typeof(GenLib.DateTimeFormatException))]
        [DataRow("10.3.2022", "xyz")]
        [DataRow("xyz", "dd. MMMM yyyy")]
        public void MethodFormatDate_IncorrectFormat_DateTimeFormatException(string date, string format)
        {
            Parameter parameter = new Parameter(format);

            MethodsLib.MethodFormatDate(date, parameter);
        }

        [TestMethod]
        public void MethodCurrentDate_CorrectDate_True()
        {
            string current = MethodsLib.MethodCurrentDate();
            string expected = DateTime.Now.ToString("dd.MM.yyyy");

            Assert.AreEqual(expected, current);
        }
    }
}
