﻿using GenLib.Entity;
using System.Collections.Generic;

namespace GenLib.Builder
{
    /// <summary>
    /// This class is Builder class for Atom entity.
    /// It provides to going through the parsing tree.
    /// </summary>
    class AtomBuilder : Builder
    {
        /// <summary>
        /// This property contains list of Atom entities.
        /// </summary>
        public List<Atom> AtomList { get; set; }

        /// <summary>
        /// Constructor of the AtomBuilder class.
        /// </summary>
        /// <param name="atoms">List of Atom entities.</param>
        public AtomBuilder(List<Atom> atoms)
        {
            AtomList = atoms;
        }

        /// <summary>
        /// This method calls going through the parsing tree for the specific 
        /// type of Atom entity descendant for each of Atom entities in the list.
        /// </summary>
        public void Build()
        {
            foreach(Atom currentAtom in AtomList)
            {
                if (currentAtom == null)
                {
                    return;
                }

                switch(currentAtom.Type)
                {
                    case EAtomType.TEXT:
                        new TextBuilder((Text) currentAtom).Build();
                        break;
                    case EAtomType.ESCAPED_CHAR:
                        new EscapedCharBuilder((EscapedChar)currentAtom).Build();
                        break;
                    case EAtomType.VARIABLE:
                        new VariableBuilder((Variable)currentAtom).Build();
                        break;
                    case EAtomType.DIRECTIVE:
                        new DirectiveBuilder((Directive)currentAtom).Build();
                        break;
                }

            }
        }
    }
}
