﻿using GenLib.Entity;
using GenLib.Entity.Enum;

namespace GenLib.Builder
{
    /// <summary>
    /// This class is Builder class for Directive entity.
    /// It provides to going through the parsing tree.
    /// </summary>
    class DirectiveBuilder : Builder
    {
        /// <summary>
        /// This property contains Directive entity.
        /// </summary>
        public Directive Directive { get; set; }

        /// <summary>
        /// Constructor of the DirectiveBuilder class.
        /// </summary>
        /// <param name="directive">Directive entity.</param>
        public DirectiveBuilder(Directive directive)
        {
            Directive = directive;
        }

        /// <summary>
        /// This method calls going through the parsing tree for the ForeachDirective 
        /// or TableForeachDirective entity of this Directive entity.
        /// </summary>
        public void Build()
        {
            if (Directive == null)
            {
                return;
            }

            switch (Directive.DirectiveType)
            {
                case EDirectiveType.FOREACH:
                    new ForeachDirectiveBuilder((ForeachDirective)Directive).Build();
                    break;
                case EDirectiveType.TABLE_FOREACH:
                    new TableForeachDirectiveBuilder((TableForeachDirective)Directive).Build();
                    break;
            }
        }
    }
}
