﻿using GenLib.Data;
using GenLib.Document;
using GenLib.Entity;

namespace GenLib.Builder
{
    /// <summary>
    /// This class is Builder class for TableForeachDirective entity.
    /// It provides to going through the parsing tree.
    /// </summary>
    class TableForeachDirectiveBuilder : Builder
    {
        /// <summary>
        /// This property contains TableForeachDirective entity.
        /// </summary>
        public TableForeachDirective TableForeachDirective { get; set; }

        /// <summary>
        /// Constructor of the TableForeachDirectiveBuilder class.
        /// </summary>
        /// <param name="tableForeachDirective">TableForeachDirective entity.</param>
        public TableForeachDirectiveBuilder(TableForeachDirective tableForeachDirective)
        {
            TableForeachDirective = tableForeachDirective;
        }

        /// <summary>
        /// This method processes TableForeachDirective entity.
        /// It inserts output table with the specific values into the final document.
        /// </summary>
        public void Build()
        {
            ControllingVariable controllingVariable = null;
            TableForeachDirective.Block.controllingVariables.TryGetValue(TableForeachDirective.ControlVariableId.Identificator, out controllingVariable);

            if (controllingVariable == null)
            {
                throw new CustomSyntaxException(ErrorType.CONTROLLING_VARIABLE_TFE_NOT_FOUND);
            }

            DataObject controllingArrayValue = DirectiveUtil.ResolveDataObject(DataRoot, TableForeachDirective.ControlArray, ControllingVariables);

            DocumentManager.Instance.AddWordRange(TableForeachDirective.ElementRange);
            DocumentManager.Instance.AddWordRange(TableForeachDirective.Block.ElementRange);

            for (int i = controllingArrayValue.Descendants.Count - 1; i >= 0; i--)
            {
                DataObject currentDataObject = controllingArrayValue.Descendants[i];

                controllingVariable.DataObject = currentDataObject;

                new BlockBuilder(TableForeachDirective.Block).Build();

                if (i != 0)
                {
                    DocumentManager.Instance.AddRowIntoTableAfterHeader(TableForeachDirective.Block.ElementRange.Range, 1);
                    DocumentManager.Instance.CopyTableValuesFromTemplate(TableForeachDirective.Block.ElementRange, TableForeachDirective.Block.ElementRange.Range);
                }
            }

            DocumentManager.Instance.CopyTable(TableForeachDirective.Block.ElementRange.Range, TableForeachDirective.ElementRange.Range.End);
            DocumentManager.Instance.DeleteRange(TableForeachDirective.ElementRange.Range);
        }
    }
}
