﻿using GenLib.Entity;
using System.Collections.Generic;

namespace GenLib.Builder
{
    /// <summary>
    /// This class is Builder class for Block entity.
    /// It provides to going through the parsing tree.
    /// </summary>
    class BlockBuilder : Builder
    {
        /// <summary>
        /// This property contains Block entity.
        /// </summary>
        public Block Block { get; set; }

        /// <summary>
        /// Constructor of the BlockBuilder class.
        /// </summary>
        /// <param name="block">Block entity.</param>
        public BlockBuilder(Block block)
        {
            Block = block;
        }

        /// <summary>
        /// This method calls going through the parsing tree 
        /// for the Atom entities of this Block entity.
        /// It adds the Block dictionary of controlling variables to the symbol table.
        /// </summary>
        public void Build()
        {
            AddControllingVariables(Block.controllingVariables);

            List<Atom> atoms = Block.Atoms;

            new AtomBuilder(atoms).Build();

            RemoveLastControllingVarsList();
        }
    }
}
