﻿using Antlr4.Runtime.Misc;
using GenLib.Data;
using GenLib.Entity;
using System.Collections.Generic;

namespace GenLib.Builder
{
    /// <summary>
    /// This abstract class contains common attributes and methods for Builder classes.
    /// It contains root of input data and symbol table for controlling variables 
    /// (for foreach and tableforeach directives).
    /// </summary>
    public abstract class Builder
    {
        /// <summary>
        /// This property is used for holding text of generated document.
        /// </summary>
        public static string FinalText { get; set; } = "";

        /// <summary>
        /// Root of the input data.
        /// </summary>
        public static DataObject DataRoot { get; set; }

        /// <summary>
        /// Symbol table for controlling variables.
        /// </summary>
        public static List<Dictionary<string, ControllingVariable>> ControllingVariables { get; set; }  = new ArrayList<Dictionary<string, ControllingVariable>>();

        /// <summary>
        /// This method append input text to final text of generated document.
        /// </summary>
        /// <param name="text">Text to append.</param>
        public static void AddText(string text)
        {
            FinalText += text;
        }

        /// <summary>
        /// This method append input character to final text of generated document.
        /// </summary>
        /// <param name="character">Character to append.</param>
        public static void AddChar(char character)
        {
            FinalText += character;
        }

        /// <summary>
        /// This method adds new dictionary of controlling variables to the symbol table.
        /// </summary>
        /// <param name="controllingVariables">Dictionary of controlling variables.</param>
        public static void AddControllingVariables (Dictionary<string, ControllingVariable> controllingVariables)
        {
            ControllingVariables.Add(controllingVariables);
        }

        /// <summary>
        /// This method removes last dictionary of controlling variables from the symbol table.
        /// </summary>
        public static void RemoveLastControllingVarsList()
        {
            ControllingVariables.RemoveAt(ControllingVariables.Count - 1);
        }
    }
}
