lexer grammar DocsGenLexer;

tokens {
  OPAR, CPAR, OBRACK, CBRACK, OBRACE, CBRACE, STRING, INTEGER, ID, REFERENCE, DOT, FOREACH, TABLEFOREACH, INCLUDE, END, K_IN
}

ESCAPED_CHAR
 : '\\' .
 ;

START_DIRECTIVE
 : '#' -> skip, pushMode(DIR_)
 ;

DOLLAR_OBRACE
 : '$' '{' -> pushMode(FRM_)
 ;

DOLLAR
 : '$' -> pushMode(VAR_)
 ;

TEXT
 : .
 ;

// Formal mode
mode FRM_;

FRM_ID
 : ID -> type(ID)
 ;

FRM_DOT
 : '.' -> type(DOT)
 ;

FRM_OPAR
 : '(' -> type(OPAR), pushMode(CODE_)
 ;

FRM_CBRACE
 : '}' -> type(CBRACE), popMode
 ;

// Directive mode
mode DIR_;

DIR_FOREACH
 : ( 'foreach' | '{foreach}' ) SPACES? '(' -> type(FOREACH), popMode, pushMode(CODE_)
 ;

DIR_TABLEFOREACH
 : ( 'tableforeach' | '{tableforeach}' ) SPACES? '(' -> type(TABLEFOREACH), popMode, pushMode(CODE_)
 ;

DIR_INCLUDE
 : ( 'include' | '{include}' ) SPACES? '(' -> type(INCLUDE), popMode, pushMode(CODE_)
 ;

DIR_END
 : ( 'end' | '{end}' ) -> type(END), popMode
 ;

DIR_CUSTOM_CODE
 : ID SPACES? '(' -> type(ID), popMode, pushMode(CODE_)
 ;

DIR_CUSTOM
 : ID -> type(ID), popMode
 ;

// Variable mode
mode VAR_;

VAR_DOLLAR
 : '$'  -> type(DOLLAR)
 ;

VAR_DOLLAR_OBRACE
 : '$' '{' -> type(DOLLAR_OBRACE), popMode, pushMode(FRM_)
 ;

VAR_HASH
 : '#' -> skip, popMode, pushMode(DIR_)
 ;

VAR_ID
 : ID -> type(ID)
 ;

VAR_DOT
 : '.' -> type(DOT)
 ;

VAR_OPAR
 : '(' -> type(OPAR), pushMode(CODE_)
 ;

VAR_TEXT
 : . -> type(TEXT), popMode
 ;

// Code mode
mode CODE_;

CODE_K_IN
 : 'in' -> type(K_IN)
 ;

CODE_ID
 : ID -> type(ID)
 ;

CODE_SPACES
 : SPACES -> skip
 ;

CODE_REFERENCE
 : '$' ID -> type(REFERENCE)
 ;

CODE_OPAR
 : '(' -> type(OPAR), pushMode(CODE_)
 ;

CODE_CPAR
 : ')' -> type(CPAR), popMode
 ;

CODE_DOT
 : '.' -> type(DOT)
 ;

CODE_INTEGER
 : INTEGER -> type(INTEGER)
 ;

CODE_STRING
 : STRING -> type(STRING)
 ;

CODE_OBRACE
 : '{' -> type(OBRACE)
 ;

CODE_CBRACE
 : '}' -> type(CBRACE)
 ;

fragment STRING
 : STRING_DQ
 | STRING_SQ
 ;

fragment SPACES    : [ \t\r\n];
fragment ID        : [a-zA-Z] [a-zA-Z0-9_-]*;
fragment STRING_DQ : '"' ( ~["\r\n] | '""' )* '"';
fragment STRING_SQ : '\'' ( ~['\r\n] | '\'\'' )* '\'';
fragment INTEGER   : DIGIT+;
fragment DIGIT     : [0-9];
fragment EXPONENT  : [eE] [+-]? DIGIT+;
