﻿using Antlr4.Runtime.Misc;
using GenLib.Entity;
using GenLib.Util;

namespace GenLib.Visitor
{
    /// <summary>
    /// This class is Visitor class for Atom entity. It provides to create parsing tree using
    /// ANTLR interface DocsGenParserBaseVisitor.
    /// </summary>
    class AtomVisitor : DocsGenParserBaseVisitor<Atom>
    {
        /// <summary>
        /// This method creates specific type of descendant of Atom entity based on the Atom entity context.
        /// Then this method calls processing of this descendants.
        /// </summary>
        /// <param name="context">Context of Atom entity.</param>
        /// <returns>New Atom entity.</returns>
        public override Atom VisitAtom([NotNull] DocsGenParser.AtomContext context)
        {
            Atom atom = null;

            if (context.TEXT() != null)
            {
                atom = new Text(context.TEXT().GetText()[0]);
            }
            else if (context.ESCAPED_CHAR() != null)
            {
                atom = new EscapedChar(context.ESCAPED_CHAR().GetText()[1]);
            }
            else if (context.variable() != null)
            {
                atom = new VariableVisitor().Visit(context.variable());
            }
            else if (context.formal() != null)
            {
                atom = new FormalVisitor().Visit(context.formal());
            }
            else if (context.property_or_method() != null)
            {
                atom = new PropertyOrMethodVisitor().Visit(context.property_or_method());
            }
            else if (context.directive() != null)
            {
                atom = new DirectiveVisitor().Visit(context.directive()); 
            }

            if (atom != null)
            {
                atom.ElementRange = ElementUtil.GetElementRange(context);
            }

            if (atom is Directive)
            {
                atom.ElementRange.Start -= 1;
            }

            return atom;
        }
    }
}
