﻿using GenLib.Entity;
using System.Collections.Generic;
using Antlr4.Runtime.Misc;
using GenLib.Util;

namespace GenLib.Visitor
{
    /// <summary>
    /// This class is Visitor class for Block entity. It provides to create parsing tree using
    /// ANTLR interface DocsGenParserBaseVisitor.
    /// </summary>
    class BlockVisitor : DocsGenParserBaseVisitor<Block>
    {
        /// <summary>
        /// This method creates Block entity and specified attributes necessary for this entity.
        /// Then this method calls processing of elements of Atom class.
        /// </summary>
        /// <param name="context">Context of Block entity.</param>
        /// <returns>New Block entity.</returns>
        public override Block VisitBlock([NotNull] DocsGenParser.BlockContext context)
        {
            List<Atom> atoms = GetAtoms(context.atom());

            Block block = new Block(atoms);

            block.ElementRange = ElementUtil.GetElementRange(context);

            return block;
        }

        /// <summary>
        /// This method gets atoms of the current Block entity.
        /// Then calls processing of the Atom entities.
        /// </summary>
        /// <param name="atomContextList">Array of contexts for Atom entities.</param>
        /// <returns>List of new Atom entities.</returns>
        private List<Atom> GetAtoms(DocsGenParser.AtomContext[] atomContextList)
        {
            List<Atom> atoms = new ArrayList<Atom>();
            Atom currentAtom;

            foreach (var atomContext in atomContextList)
            {
                currentAtom = new AtomVisitor().Visit(atomContext);
                atoms.Add(currentAtom);
            }

            return atoms;
        }
    }
}
