﻿using Antlr4.Runtime.Misc;
using GenLib.Entity;

namespace GenLib.Visitor
{
    /// <summary>
    /// This class is Visitor class for 'directive' rule of ANTLR grammar. It provides to create
    /// parsing tree using ANTLR interface DocsGenParserBaseVisitor.
    /// </summary>
    class DirectiveVisitor : DocsGenParserBaseVisitor<Directive>
    {
        /// <summary>
        /// This method creates Directive entity and specified attributes necessary for this entity
        /// withing the parsing of 'directive' rule of ANTLR grammar. Then this method
        /// calls processing of 'foreach_directive' or 'tableforeach_directive' rule of ANTLR grammar.
        /// </summary>
        /// <param name="context">Context of 'directive' rule.</param>
        /// <returns>New Directive entity.</returns>
        public override Directive VisitDirective([NotNull] DocsGenParser.DirectiveContext context)
        {
            Directive directive = null;

            if (context.foreach_directive() != null)
            {
                directive = new ForeachDirectiveVisitor().Visit(context.foreach_directive());
            }
            else if (context.tableforeach_directive() != null)
            {
                directive = new TableForeachDirectiveVisitor().Visit(context.tableforeach_directive());
            }

            return directive;
        }
    }
}
