﻿using Antlr4.Runtime.Misc;
using GenLib.Entity;

namespace GenLib.Visitor
{
    /// <summary>
    /// This class is Visitor class for 'foreach_directive' rule of ANTLR grammar. It provides to create
    /// parsing tree using ANTLR interface DocsGenParserBaseVisitor.
    /// </summary>
    class ForeachDirectiveVisitor : DocsGenParserBaseVisitor<ForeachDirective>
    {
        /// <summary>
        /// This method creates ForeachDirective entity and specified attributes necessary for this entity
        /// withing the parsing of 'foreach_directive' rule of ANTLR grammar. Then this method
        /// calls processing of 'variable', 'property_or_method' and 'block' rules of ANTLR grammar.
        /// </summary>
        /// <param name="context">Context of 'foreach_directive' rule.</param>
        /// <returns>New ForeachDirective entity.</returns>
        public override ForeachDirective VisitForeach_directive([NotNull] DocsGenParser.Foreach_directiveContext context)
        {
            Variable controlVariable = new VariableVisitor().Visit(context.variable());
            Variable controlArray = new PropertyOrMethodVisitor().Visit(context.property_or_method());
            Block block = new BlockVisitor().Visit(context.block());
            block.controllingVariables.Add(controlVariable.Identificator, new ControllingVariable(controlVariable.Identificator));
            
            return new ForeachDirective(controlVariable, controlArray, block);
        }
    }
}
