﻿using Antlr4.Runtime.Misc;
using GenLib.Entity;

namespace GenLib.Visitor
{
    /// <summary>
    /// This class is Visitor class for 'tableforeach_directive' rule of ANTLR grammar. It provides to create
    /// parsing tree using ANTLR interface DocsGenParserBaseVisitor.
    /// </summary>
    class TableForeachDirectiveVisitor : DocsGenParserBaseVisitor<TableForeachDirective>
    {
        /// <summary>
        /// This method creates TableForeachDirective entity and specified attributes necessary for this entity
        /// withing the parsing of 'tableforeach_directive' rule of ANTLR grammar. Then this method
        /// calls processing of 'variable', 'property_or_method' and 'block' rules of ANTLR grammar.
        /// </summary>
        /// <param name="context">Context of 'tableforeach_directive' rule.</param>
        /// <returns>New TableForeachDirective entity.</returns>
        public override TableForeachDirective VisitTableforeach_directive([NotNull] DocsGenParser.Tableforeach_directiveContext context)
        {
            Variable controlVariable = new VariableVisitor().Visit(context.variable());
            Variable controlArray = new PropertyOrMethodVisitor().Visit(context.property_or_method());
            Block block = new BlockVisitor().Visit(context.block());
            block.controllingVariables.Add(controlVariable.Identificator, new ControllingVariable(controlVariable.Identificator));

            return new TableForeachDirective(controlVariable, controlArray, block);
        }
    }
}
