﻿using Antlr4.Runtime.Misc;
using GenLib.Entity;

namespace GenLib.Visitor
{
    /// <summary>
    /// This class is Visitor class for 'formal' rule of ANTLR grammar. It provides to create 
    /// parsing tree using ANTLR interface DocsGenParserBaseVisitor.
    /// </summary>
    class FormalVisitor : DocsGenParserBaseVisitor<Variable>
    {
        /// <summary>
        /// This method creates Variable entity and specified attributes necessary for this entity
        /// withing the parsing of 'formal' rule of ANTLR grammar. Then this method calls processing
        /// of 'formal_property_or_method' rule of ANTLR grammar.
        /// </summary>
        /// <param name="context">Context of 'formal' rule.</param>
        /// <returns>New Variable entity.</returns>
        public override Variable VisitFormal([NotNull] DocsGenParser.FormalContext context)
        {
            if (context.id() != null && !context.id().IsEmpty)
            {
                string identificator = context.id().GetText();
                return new Variable(identificator);
            }
            else if (context.formal_property_or_method() != null && !context.formal_property_or_method().IsEmpty)
            {
                return new FormalPropertyOrMethodVisitor().Visit(context.formal_property_or_method());
            }
            else
            {
                throw new CustomSyntaxException(ErrorType.INCONSISTENT_FORMAL_VISITOR);
            }

        }
    }
}
