﻿using DocsGen.Services;
using System;
using System.Windows.Forms;
using TemplateBuilder.Model.Entities.GeneralObjects.GeneralObjectMethods;

namespace TemplateBuilder.Forms
{
    /// <summary>
    /// This partial class serves actions for CurrentDateForm. 
    /// The form mediates GUI for the Current Date method.
    /// </summary>
    public partial class CurrentDateForm : Form
    {
        /// <summary>
        /// Constructor of the CurrentDateForm class.
        /// </summary>
        public CurrentDateForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// This method serves action on storno button click in CurrentDateForm.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void StornoCurrentDateButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// This method serves action on add button click in CurrentDateForm.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void AddCurrentDateButton_Click(object sender, EventArgs e)
        {
            PrintCurrentDateMark();
        }

        /// <summary>
        /// This method provides to resolve language construction of the Current Date method
        /// and to add the language construction into Word document template.
        /// </summary>
        private void PrintCurrentDateMark()
        {
            GeneralObjectCurrentDateMethod currentDateMethod;

            try
            {
                if (formatCurrentDateCheckBox.Checked)
                {
                    currentDateMethod = new GeneralObjectCurrentDateMethod(formatCurrentDateTextBox.Text);
                }
                else
                {
                    currentDateMethod = new GeneralObjectCurrentDateMethod();
                }

                WordManager.InsertMark(currentDateMethod.ToString());
                Close();
            }
            catch(Exception e)
            {

                string infoMessage = "Adding Format Date method failed.\nError:\n" + e.Message;
                MessageBox.Show(infoMessage, "Error: Format Date method", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// This method serves action checked format current 
        /// date check box changed in CurrentDateForm.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void FormatCurrentDateCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkBox = sender as CheckBox;

            if (checkBox.Checked)
            {
                formatCurrentDateTextBox.Enabled = true;
            }
            else
            {
                formatCurrentDateTextBox.Enabled = false;
            }
        }
    }
}
