﻿using DocsGen.Services;
using System;
using System.IO;
using System.Windows.Forms;
using TemplateBuilder.Model.Entities.GeneralObjects.GeneralObjectDirectives;

namespace TemplateBuilder.Forms
{
    /// <summary>
    /// This partial class serves actions for IncludeForm. 
    /// The form mediates GUI for the include directive.
    /// </summary>
    public partial class IncludeForm : Form
    {
        /// <summary>
        /// Instance of global WinForms dialogs.
        /// </summary>
        private Dialogs _dialogs = null;

        /// <summary>
        /// Sub template file path.
        /// </summary>
        private string _filePath = null;

        /// <summary>
        /// Constructor of the IncludeForm class.
        /// </summary>
        public IncludeForm()
        {
            _dialogs = Globals.ThisAddIn.Dialogs;
            InitializeComponent();
        }

        /// <summary>
        /// This method serves action on storno button click in IncludeForm.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void StornoIncludeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        /// <summary>
        /// This method serves action on choose data file button click in IncludeForm.
        /// It provides to load sub template file.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void ChooseFileButton_Click(object sender, EventArgs e)
        {
            _filePath = _dialogs.IncludeFileChooseDialog();
            string fileName = Path.GetFileName(_filePath);

            chooseFileLabel.Text = fileName;
            addIncludeButton.Enabled = true;
        }

        /// <summary>
        /// This method serves action on add button click in IncludeForm.
        /// It provides to resolve language construction of include directive
        /// and to add the language construction into Word document template.
        /// </summary>
        /// <param name="sender">Sender.</param>
        /// <param name="e">Event arguments.</param>
        private void AddIncludeButton_Click(object sender, EventArgs e)
        {
            if (_filePath != null)
            {
                PrintIncludeMark();
                Close();
            }
            else
            {
                string errorMessage = "Include file path not specified. Specify the path and try it again.";
                MessageBox.Show(errorMessage, "Error: Path not specified", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }            
        }

        /// <summary>
        /// This method provides to resolve language construction of include directive
        /// and to add the language construction into Word document template.
        /// </summary>
        private void PrintIncludeMark()
        {
            GeneralObjectIncludeDirective includeDirective = new GeneralObjectIncludeDirective(_filePath);

            WordManager.InsertMark(includeDirective.ToString());
        }
    }
}
