﻿namespace TemplateBuilder.Model.Entities.Metadata
{
    /// <summary>
    /// This class contains metadata parameters for needs of metadata tree view.
    /// </summary>
    public class MetadataParams
    {
        /// <summary>
        /// Specific type of MetadataParams: MetadataAttribute or MetadataObject.
        /// </summary>
        public object MetadataClass { get; private set; }

        /// <summary>
        /// Controlling variable for foreach or tableforeach directive.
        /// </summary>
        public string ControlVariableId { get; private set; }

        /// <summary>
        /// Controlling array for foreach or tableforeach directive.
        /// </summary>
        public string ControlArray { get; private set; }

        /// <summary>
        /// Constructor of the MetadataParams class.
        /// </summary>
        /// <param name="metadataClass">Specific type of MetadataParams: MetadataAttribute or MetadataObject.</param>
        public MetadataParams(object metadataClass) : this(metadataClass, null, null) { }

        /// <summary>
        /// Constructor of the MetadataParams class.
        /// </summary>
        /// <param name="metadataClass">Specific type of MetadataParams: MetadataAttribute or MetadataObject.</param>
        /// <param name="controlVariableId">Controlling variable for foreach or tableforeach directive.</param>
        /// <param name="controlArray">Controlling array for foreach or tableforeach directive.</param>
        public MetadataParams(object metadataClass, string controlVariableId, string controlArray)
        {
            MetadataClass = metadataClass;
            ControlVariableId = controlVariableId;
            ControlArray = controlArray;
        }
    }
}
