﻿using TemplateBuilder.Model.Entities.Metadata;

namespace TemplateBuilder.Model.Entities.NodeObjects
{
    /// <summary>
    /// This abstract class contains common attributes and methods 
    /// for NodeObject descendants.
    /// </summary>
    public abstract class NodeObject
    {
        /// <summary>
        /// Specific type of node object.
        /// </summary>
        public ENodeObjectType ObjectType { get; set; }

        /// <summary>
        /// Full name of node object.
        /// </summary>
        public string NodeObjectFullName { get; set; }

        /// <summary>
        /// Label of the current node object.
        /// </summary>
        public string Label { get; set; }
        
        /// <summary>
        /// Language construction of current node object to insert into document template.
        /// </summary>
        public abstract string TemplateMark { get; }

        /// <summary>
        /// Constructor of the NodeObject class.
        /// </summary>
        /// <param name="nodeObjectFullName">Full name of the node object.</param>
        /// <param name="label">Label of the current node object.</param>
        public NodeObject(string nodeObjectFullName, string label)
        {
            NodeObjectFullName = nodeObjectFullName;
            Label = label;
        }

        /// <summary>
        /// This method returns label of the current node object.
        /// </summary>
        /// <returns>Label of the current node object.</returns>
        public override string ToString()
        {
            return Label;
        }
    }
}
