﻿using System.Text.RegularExpressions;

namespace TemplateBuilder.Model.Entities.NodeObjects.Directives
{
    /// <summary>
    /// This class represents foreach directive in metadata tree view.
    /// </summary>
    public class NodeObjectForeachDirective : NodeObjectDirective
    {
        /// <summary>
        /// Default value of controlling variable.
        /// </summary>
        private static readonly string DEFAULT_CONTROLLING_VARIABLE = "variable";
        
        /// <summary>
        /// This attribute contains controlling variable of the foreach directive.
        /// </summary>
        private string controllingVariable;

        /// <summary>
        /// Property for controllingVariable attribute.
        /// </summary>
        private string ControllingVariable 
        {
            get
            {
                return controllingVariable;
            } 
            set
            {
                string variable = value;

                variable = Regex.Replace(variable, @"\s+", string.Empty);
                
                if (variable == null || variable.Equals(string.Empty))
                {
                    controllingVariable = DEFAULT_CONTROLLING_VARIABLE;
                }
                else
                {
                    controllingVariable = variable;
                }
            }
        }

        /// <summary>
        /// Full name of the node object.
        /// </summary>
        private string NodeObjectFullName { get; set; }

        /// <summary>
        /// Constructor of the NodeObjectForeachDirective class.
        /// </summary>
        /// <param name="controllingVariable">Controlling variable of the foreach directive.</param>
        /// <param name="nodeObjectFullName">Full name of the node object.</param>
        public NodeObjectForeachDirective(string controllingVariable, string nodeObjectFullName)
        {
            ControllingVariable = controllingVariable;
            NodeObjectFullName = nodeObjectFullName;
            DirectiveType = ENodeObjectDirectiveType.Foreach;
        }

        /// <summary>
        /// This method returns language construction of the foreach directive.
        /// </summary>
        /// <returns>Language construction of the foreach directive.</returns>
        public override string ToString()
        {
            return "#foreach ($" + ControllingVariable + " in $" + NodeObjectFullName + ")\n\n#end";
        }
    }
}
