﻿using System;

namespace TemplateBuilder.Model.Entities.NodeObjects.Methods
{
    /// <summary>
    /// This class represents Format Date method in metadata tree view.
    /// </summary>
    public class NodeObjectFormatDateMethod : NodeObjectMethod
    {
        /// <summary>
        /// This attribute is parameter of the current Format Date method.
        /// </summary>
        private string parameter;

        /// <summary>
        /// Property for the 'parameter' attribute.
        /// </summary>
        private string Parameter
        {
            get
            {
                return parameter;
            }
            set
            {
                string param = value;

                if (IsParameterValid(param))
                {
                    parameter = param;
                }
                else
                {
                    throw new DateTimeFormatException(ErrorType.DATETIME_FORMAT_NOT_VALID + " Entered format: " + param);
                }
            }
        }

        /// <summary>
        /// Constructor of the NodeObjectFormatDateMethod class.
        /// </summary>
        /// <param name="parameter">Parameter of the Format Date method.</param>
        public NodeObjectFormatDateMethod(string parameter)
        {
            Parameter = parameter;
            MethodType = ENodeObjectMethodType.FormatDate;
        }

        /// <summary>
        /// This method return language construction of the current Format Date method.
        /// </summary>
        /// <returns>Language construction of the current Format Date method.</returns>
        public override string ToString()
        {
            return ".formatDate(\"" + Parameter + "\")";
        }

        /// <summary>
        /// This method checks if Current Date method parameter is valid.
        /// </summary>
        /// <param name="parameter">Current Date method parameter.</param>
        /// <returns>true - if parameter is valid; false - otherwise.</returns>
        private bool IsParameterValid(string parameter)
        {
            if (parameter.Equals(string.Empty))
            {
                return false;
            }

            DateTime date = DateTime.Now;
            string convertedDateString = date.ToString(parameter);

            if (!DateTime.TryParse(convertedDateString, out date))
            {
                return false;
            }

            return true;
        }
    }
}
