﻿using Word = Microsoft.Office.Interop.Word;
using TemplateBuilder;

namespace DocsGen.Services
{
    /// <summary>
    /// This static class contains methods for work with Word documents.
    /// </summary>
    public static class WordManager
    {
        /// <summary>
        /// This method inserts table with initial values (for needs of tableforeach directive) into document template.
        /// </summary>
        /// <param name="headerRowValues">Values of 'header row'.</param>
        /// <param name="valueRowValues">Values of 'value row'.</param>
        public static void InsertForeachTableWithValues(string[] headerRowValues, string[] valueRowValues)
        {
            if (headerRowValues.Length != valueRowValues.Length)
            {
                throw new AttributeException(ErrorType.INCONSISTENT_TABLE);
            }

            Word.Application application = Globals.ThisAddIn.Application;
            Word.Selection currentSelection = application.Selection;
            Word.Range range = currentSelection.Range;

            Word.Table newTable = currentSelection.Tables.Add(range, 2, headerRowValues.Length);
            newTable.Borders.InsideLineStyle = Word.WdLineStyle.wdLineStyleSingle;
            newTable.Borders.OutsideLineStyle = Word.WdLineStyle.wdLineStyleSingle;
            newTable.AllowAutoFit = true;

            for (int i = 1; i <= newTable.Columns.Count; i++)
            {
                newTable.Cell(1, i).Range.Text = headerRowValues[i - 1];
                newTable.Cell(2, i).Range.Text = valueRowValues[i - 1];
            }

            application.ActiveDocument.Range(newTable.Range.End, newTable.Range.End).Select();
        }

        /// <summary>
        /// This method inserts language construction into document template.
        /// </summary>
        /// <param name="markToInsert">Language construction to insert.</param>
        public static void InsertMark(string markToInsert)
        {
            Word.Application application = Globals.ThisAddIn.Application;
            Word.Selection currentSelection = application.Selection;

            bool userOvertype = application.Options.Overtype;

            if (Globals.ThisAddIn.Application.Options.Overtype)
            {
                Globals.ThisAddIn.Application.Options.Overtype = false;
            }

            if (currentSelection.Type == Word.WdSelectionType.wdSelectionIP)
            {
                currentSelection.TypeText(markToInsert);
            }

            Globals.ThisAddIn.Application.Options.Overtype = userOvertype;
        }

    }
}
